@@ -23,6 +23,7 @@ Puppet::Type.type(:glance_image).provide(
 
   def create
     temp_file = false
+
     if @resource[:source]
       if @resource[:proxy]
         proxy_uri = URI(@resource[:proxy])
@@ -38,30 +39,17 @@ Puppet::Type.type(:glance_image).provide(
         location = "--file=#{@resource[:source]}"
       else
         temp_file = Tempfile.new('puppet-glance-image')
-
-        uri = URI(@resource[:source])
-        Net::HTTP.start(uri.host, uri.port, proxy_host, proxy_port,
-                        :use_ssl => uri.scheme == 'https') do |http|
-          request = Net::HTTP::Get.new uri
-          http.request request do |response|
-            open temp_file.path, 'w' do |io|
-              response.read_body do |segment|
-                io.write(segment)
-              end
-            end
-          end
+        response = fetch(@resource[:source], proxy_host, proxy_port)
+        open temp_file.path, 'w' do |io|
+            io.write(response.read_body)
         end
 
         location = "--file=#{temp_file.path}"
       end
-
-    # location cannot handle file://
-    # location does not import, so no sense in doing anything more than this
-    elsif @resource[:location]
-      location = "--location=#{@resource[:location]}"
     else
-      raise(Puppet::Error, "Must specify either source or location")
+      raise(Puppet::Error, "The source parameter is required to create an image")
     end
+
     opts = [@resource[:name]]
 
     opts << (@resource[:is_public] == :true ? '--public' : '--private')
@@ -74,9 +62,28 @@ Puppet::Type.type(:glance_image).provide(
     opts << "--tag=#{@resource[:image_tag]}" if @resource[:image_tag]
     opts << location
 
+    if @resource[:project_name]
+      opts << "--project=#{@resource[:project_name]}"
+    elsif @resource[:project_id]
+      opts << "--project=#{@resource[:project_id]}"
+    end
+
     begin
-      @property_hash = self.class.request('image', 'create', opts)
-      @property_hash[:ensure] = :present
+      attrs = self.class.request('image', 'create', opts)
+      properties = self.class.parse_python_dict(attrs[:properties])
+      @property_hash = {
+        :ensure           => :present,
+        :name             => attrs[:name],
+        :is_public        => attrs[:visibility].downcase.chomp == 'public'? true : false,
+        :container_format => attrs[:container_format],
+        :id               => attrs[:id],
+        :disk_format      => attrs[:disk_format],
+        :min_disk         => attrs[:min_disk],
+        :min_ram          => attrs[:min_ram],
+        :properties       => self.class.exclude_owner_specified_props(self.class.exclude_readonly_props(properties)),
+        :image_tag        => attrs[:tags],
+        :project_id       => attrs[:owner]
+      }
     ensure
       if temp_file
         temp_file.close(true)
@@ -95,36 +102,24 @@ Puppet::Type.type(:glance_image).provide(
 
   mk_resource_methods
 
-  def is_public=(value)
-    @property_flush[:is_public] = value
-  end
-
   def is_public
     bool_to_sym(@property_hash[:is_public])
   end
 
-  def disk_format=(value)
-    @property_flush[:disk_format] = value
-  end
-
-  def container_format=(value)
-    @property_flush[:container_format] = value
-  end
-
-  def min_ram=(value)
-    @property_flush[:min_ram] = value
-  end
-
-  def min_disk=(value)
-    @property_flush[:min_disk] = value
-  end
-
-  def properties=(value)
-    @property_flush[:properties] = value
-  end
-
-  def image_tag=(value)
-    @property_flush[:image_tag] = value
+  [
+    :is_public,
+    :disk_format,
+    :container_format,
+    :min_ram,
+    :min_disk,
+    :properties,
+    :image_tag,
+    :project_id,
+    :project_name
+  ].each do |attr|
+    define_method(attr.to_s + "=") do |value|
+      @property_flush[attr] = value
+    end
   end
 
   def id=(id)
@@ -135,7 +130,7 @@ Puppet::Type.type(:glance_image).provide(
     list = request('image', 'list', '--long')
     list.collect do |image|
       attrs = request('image', 'show', image[:id])
-      properties = parsestring(attrs[:properties]) rescue nil
+      properties = parse_python_dict(attrs[:properties])
       new(
         :ensure           => :present,
         :name             => attrs[:name],
@@ -146,7 +141,8 @@ Puppet::Type.type(:glance_image).provide(
         :min_disk         => attrs[:min_disk],
         :min_ram          => attrs[:min_ram],
         :properties       => exclude_owner_specified_props(exclude_readonly_props(properties)),
-        :image_tag        => attrs[:image_tag]
+        :image_tag        => attrs[:tags],
+        :project_id       => attrs[:owner]
       )
     end
   end
@@ -173,6 +169,12 @@ Puppet::Type.type(:glance_image).provide(
       (opts << props_to_s(@property_flush[:properties])) if @property_flush[:properties]
       (opts << "--tag=#{@property_flush[:image_tag]}") if @property_flush[:image_tag]
 
+      if @property_flush[:project_name]
+        opts << "--project=#{@property_flush[:project_name]}"
+      elsif @property_flush[:project_id]
+        opts << "--project=#{@property_flush[:project_id]}"
+      end
+
       self.class.request('image', 'set', opts)
       @property_flush.clear
     end
@@ -181,18 +183,19 @@ Puppet::Type.type(:glance_image).provide(
   private
 
   def self.exclude_readonly_props(props)
-    if props == nil
-      return nil
-    end
-    hidden = ['os_hash_algo', 'os_hash_value', 'os_hidden']
+    hidden = [
+      'os_hash_algo',
+      'os_hash_value',
+      'os_hidden',
+      'direct_url',
+      'stores',
+      'locations'
+    ]
     rv = props.select { |k, v| not hidden.include?(k) }
     return rv
   end
 
   def self.exclude_owner_specified_props(props)
-    if props == nil
-      return nil
-    end
     rv = props.select { |k, v| not k.start_with?('owner_specified.') }
     return rv
   end
@@ -204,21 +207,24 @@ Puppet::Type.type(:glance_image).provide(
     }.compact
   end
 
-  def self.string2hash(input)
-    return Hash[input.scan(/(\S+)='([^']*)'/)]
-  end
+  def fetch(uri_str, proxy_host = nil, proxy_port = nil, limit = 10)
+    raise RuntimeError, 'Too many HTTP redirections' if limit == 0
 
-  def self.pythondict2hash(input)
-    return JSON.parse(input.gsub(/u'(\w*)'/, '"\1"').gsub(/'/, '"').gsub(/False/,'false').gsub(/True/,'true'))
-  end
-
-  def self.parsestring(input)
-    if input[0] == '{'
-      # 4.0.0+ output, python dict
-      return self.pythondict2hash(input)
-    else
-      # Pre-4.0.0 output, key=value
-      return self.string2hash(input)
+    uri = URI(uri_str)
+    Net::HTTP.start(uri.host, uri.port, proxy_host, proxy_port,
+                    :use_ssl => uri.scheme == 'https') do |http|
+      request = Net::HTTP::Get.new uri
+      http.request request do |response|
+        case response
+        when Net::HTTPSuccess then
+          response
+        when Net::HTTPRedirection then
+          location = response['location']
+          return fetch(location, proxy_host, proxy_port, limit - 1)
+        else
+          response.value
+        end
+      end
     end
   end
 end
